#include <math.h>
#include "pub_s.hh"
#include "util_c.hh"
#include "dispatch.h"
#include "vport.h"

#define RAND rand()/(pow(2, 31) -1)

class Server: public _sk_Publisher
{
    private:
	SubscriberList		_subscribers;

	void	append(Subscriber_ptr s) {
		CORBA::ULong len = _subscribers.length();
		_subscribers.length(len+1);
		_subscribers[len] = Subscriber::_duplicate(s);
	}
	void	remove(CORBA::ULong idx) {
		CORBA::ULong len = _subscribers.length();
		if ( idx < len) {
			CORBA::ULong i;
			for ( i=idx; i < len-1; i++)
				_subscribers[i] = Subscriber::_duplicate(
					_subscribers[i+1]);
			_subscribers[i] = Subscriber::_nil();
			_subscribers.length(len-1);
		}
	}
	CORBA::ULong	index(Subscriber_ptr s) {
		CORBA::ULong len = _subscribers.length();
		CORBA::ULong i;
		for (i=0; i< len; i++) {
			if (_subscribers[i]->_is_equivalent(s) )
				break;
		}
		return i;
	}

    public:
	Server(const char *name=NULL) : _sk_Publisher(name) {}
	~Server() {}

	void 	add_subscriber(Subscriber_ptr s) {
		cout << "Adding new subscriber" << endl;
		append(s);
	}
        void	remove_subscriber(Subscriber_ptr s) {
		cout << "Removing subscriber" << endl;
		remove(index(s));
	}
        void	publish(const Quote& q);
  	SubscriberList* get_subscribers();
};


void Server::publish(const Quote& q)
{
	CORBA::ULong len = _subscribers.length();
	for ( ; len >0; len--) {
		try {
			_subscribers[len-1]->receive_update(q);
		}
		catch(const CORBA::Exception& excep) {
			cout << "Subscriber's receive update failed" << endl;
			cout << "Subscriber number " << len-1 << endl;
  			cout << excep << endl;
			remove_subscriber(_subscribers[len-1]);
		}
	}
}

SubscriberList* Server::get_subscribers()
{
	return SubscriberList::_duplicate(&_subscribers);
}

const char *names[5] = {
	"IBM",
	"DEC",
	"HP",
	"SUN",
	"SCO" };


int main(int argc, char *const *argv)
{
	CORBA::ORB_var orb = CORBA::ORB_init(argc, argv);
	CORBA::BOA_var boa = orb->BOA_init(argc, argv);

	Server server("StockQuotePublisher");
	boa->obj_is_ready(&server);


	for(CORBA::ULong i=1;; i++) {
		CORBA::ULong idx = i % 5;
		Quote q;
		q.name = (const char *)names[idx];
		q.value = 50.0 + RAND * 40.0;
		server.publish(q);
		
#if defined(THREAD)
		VISPortable::vsleep(2);
#else
		// Sleep for two seconds and allow the
		// ORB to process its messages
		long sec=2, msec=0;
		Dispatcher::instance().dispatch(sec,msec);
#endif
	}

	return 0;
}
	
